#!/usr/bin/env python
# -*- coding: utf-8 -*-
import uuid
import logging
from logging.handlers import RotatingFileHandler

from flask import abort, jsonify
from flask_cors import CORS
from flask_compress import Compress
import six
import six.moves.urllib.parse as urlparse

from coshed.swagger_glue import next_best_swagger_in_source
from coshed.tools import load_json

#: custom log formatter
formatter = logging.Formatter(
    fmt='%(asctime)s %(name)-16s %(levelname)-8s %(funcName)-15s '
        '(#%(lineno)04d): %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S')


def rotating_app_log(flask_app_instance, app_name=None):
    if app_name is None:
        app_name = uuid.uuid4().hex

    log_filename = '/tmp/{:s}-webapp.log'.format(app_name)
    handler = RotatingFileHandler(log_filename, maxBytes=10000, backupCount=1)
    handler.setLevel(logging.DEBUG)
    handler.setFormatter(formatter)
    flask_app_instance.logger.addHandler(handler)
    flask_app_instance.logger.setLevel(logging.INFO)
    logging.getLogger("iotta.boarding_queue").addHandler(handler)

    return log_filename


def wolfication(flask_app_instance=None, jinja_filters=None, app_name=None):
    flask_app_instance.jinja_env.trim_blocks = True
    flask_app_instance.jinja_env.lstrip_blocks = True
    flask_app_instance.jinja_env.strip_trailing_newlines = True
    flask_app_instance.config['SEND_FILE_MAX_AGE_DEFAULT'] = 24 * 3600 * 365
    flask_app_instance.config['MAX_CONTENT_LENGTH'] = 2 * 1024 * 1024

    if jinja_filters is not None:
        for (filter_key, filter_func) in six.iteritems(jinja_filters):
            flask_app_instance.jinja_env.filters[filter_key] = filter_func

    CORS(flask_app_instance)
    Compress(flask_app_instance)

    if app_name is not None:
        rotating_app_log(flask_app_instance, app_name)

    return flask_app_instance


def serve_swagger_handler(swagger_root_path, app_name=None, overrides=None):
    """
    Load swagger specification, apply overrides and return Flask HTTP
    response object.

    Args:
        swagger_root_path: swagger files path
        app_name: application name
        overrides (dict): specification overrides

    Returns:
        Flask.Response: HTTP response object
    """
    try:
        swagger_in = next_best_swagger_in_source(swagger_root_path,
                                                 app_name=app_name)
    except ValueError:
        abort(404)

    swagger_content = load_json(swagger_in)
    if overrides is not None:
        swagger_content.update(overrides)

    return jsonify(swagger_content)



# !/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import logging
import subprocess


class SassyController(object):
    __slots__ = ['log', 'item_tuples']

    def __init__(self, *args, **kwargs):
        self.log = logging.getLogger(__name__)
        self.item_tuples = []

        if args and len(args) >= 2:
            self.add_item(args[0], args[1])
        else:
            self.add_item(
                (kwargs.get("source_path"), kwargs.get("target_path")))

    def add_item(self, source_path, target_path):
        try:
            self.item_tuples.append(
                (os.path.normpath(source_path), os.path.normpath(target_path))
            )
        except AttributeError as aexc:
            self.log.error("Bad source/target parameter: {!r}".format(
                (source_path, target_path)))
            raise

    def run(self, force=False):
        for source_path, target_path in self.item_tuples:
            for source, target in self.sassy_items(source_path, target_path,
                                                   force=force):
                args = [
                    'sassc',
                    '-t',
                    'compressed',
                    '--omit-map-comment',
                    source,
                    target
                ]
                self.log.debug("Calling {!r}".format(args))
                result = subprocess.check_call(args)
                self.log.debug(result)

    def sassy_items(self, source_path, target_path, force=False):
        for item in os.listdir(source_path):
            if item.startswith('_'):
                continue

            (trunk, ext) = os.path.splitext(item)

            if ext.lower() not in ('.scss', '.sass'):
                continue

            source = os.path.normpath(
                os.path.join(source_path, item)
            )
            target = os.path.normpath(
                os.path.join(target_path, '{:s}.css'.format(trunk))
            )
            do_yield = False

            self.log.info(item)

            if not os.path.isfile(target):
                do_yield = True
            else:
                source_stat = os.stat(source)
                target_stat = os.stat(target)

                if source_stat.st_mtime > target_stat.st_mtime:
                    self.log.info("Need update!")
                    do_yield = True

            if do_yield:
                yield source, target


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    sc = SassyController("./sassy", './static/css')
    sc.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging

import pendulum

try:
    from transitions import Machine
except ImportError:
    print("I need the transitions package but will fail later.")

STATE_S0 = 's0'
STATE_OPEN = 'open'
STATE_CLOSED = 'closed'
STATE_DEAD = 'dead'


class StretchyMachineException(Exception):
    pass


class StretchyMachineStillAlive(StretchyMachineException):
    pass


class StretchyMachine(Machine):
    def dump_state_of_the_onion(self):
        self.log.debug(
            "state={state:6s} current_value={current_value!s:16} "
            "#dt_objects={dt_objects_len:5d} #periods={periods_len:5d}".format(
                state=self.state, current_value=self.current_value,
                dt_objects_len=len(self.dt_objects),
                periods_len=len(self.periods)
            ))

    def on_enter_open(self, *args, **kwargs):
        self.log.debug("ENTER OPEN")
        self.current_value = kwargs.get(self.value_key)
        dt_object = kwargs.get(self.dt_key)

        if dt_object:
            self.log.debug(
                "+ {!r:10} {!r}".format(self.current_value, dt_object))

            if len(self.dt_objects):
                if dt_object <= self.dt_objects[-1]:
                    self.log.warning(
                        "{!r} <= {!r}".format(dt_object, self.dt_objects[-1]))
            self.dt_objects.append(dt_object)

        self.dump_state_of_the_onion()
        self.log.debug('')

    def on_exit_open(self, *args, **kwargs):
        self.log.debug("# EXIT OPEN")
        self.overhang_dt_object = kwargs.get(self.dt_key)
        if self.overhang_dt_object:
            self.log.debug(
                "= {!r:10} {!r}".format(self.current_value,
                                        self.overhang_dt_object))

    def _terminate_period(self):
        try:
            current_p = pendulum.Period(self.dt_objects[0], self.dt_objects[-1])
            self.periods.append((current_p, self.current_value))
        except Exception as exc:
            self.log.debug(exc)

        self.dt_objects = []

    def on_enter_dead(self, *args, **kwargs):
        self.log.debug("DEAD")
        self._terminate_period()
        self.periods = sorted(self.periods)
        self.dump_state_of_the_onion()

    def on_exit_closed(self, *args, **kwargs):
        self.log.debug("EXIT CLOSED")
        self.log.debug("~" * 80)
        self.dump_state_of_the_onion()
        self._terminate_period()
        self.current_value = kwargs.get(self.value_key)

        if self.overhang_dt_object:
            self.log.debug(
                "! {!r:10} {!r}".format(self.current_value,
                                        self.overhang_dt_object))
            self.dt_objects.append(self.overhang_dt_object)
            self.overhang_dt_object = None

        self.log.debug("v" * 80)
        self.dump_state_of_the_onion()
        self.log.debug('')

    def changed_value(self, *args, **kwargs):
        is_changed = self.current_value != kwargs.get(self.value_key)
        return is_changed

    def same_value(self, *args, **kwargs):
        return not self.changed_value(*args, **kwargs)

    def have_payload(self, *args, **kwargs):
        for key in self.required_payload_keys:
            try:
                kwargs[key]
            except KeyError:
                return False

        return True

    def yield_stretches_with(self, value, auto_terminate=True):
        if self.state != STATE_DEAD:
            if auto_terminate:
                self.terminate()
            else:
                raise StretchyMachineStillAlive("I am not terminated!")

        for (period, p_value) in self.periods:
            if p_value == value:
                yield period

    def __init__(self, *args, **kwargs):
        states = [STATE_S0, STATE_OPEN, STATE_CLOSED, STATE_DEAD]

        Machine.__init__(self, states=states, initial=STATE_S0)
        self.log = logging.getLogger(__name__)
        self.periods = []
        self.dt_objects = []
        self.current_value = None
        self.overhang_dt_object = None
        self.dt_key = kwargs.get("dt_key", "dt")
        self.value_key = kwargs.get("value_key", "value")
        self.required_payload_keys = (self.dt_key, self.value_key)

        self.add_transition('shovel', STATE_S0, STATE_OPEN,
                            conditions=['have_payload'])

        self.add_transition('terminate', '*', STATE_DEAD)

        self.add_transition('shovel', STATE_OPEN, STATE_CLOSED,
                            conditions=['have_payload', 'changed_value'])

        self.add_transition('shovel', STATE_OPEN, STATE_OPEN,
                            conditions=['have_payload', 'same_value'])

        self.add_transition('shovel', STATE_CLOSED, STATE_OPEN,
                            conditions=['have_payload', 'changed_value'])


if __name__ == '__main__':
    logging.basicConfig(
        # level=logging.DEBUG,
        level=logging.INFO,
        format='%(asctime)s %(levelname)-8s %(message)s',
        datefmt='%Y%m%d %H:%M:%S')

    logging.getLogger('transitions').setLevel(logging.FATAL)
    m = StretchyMachine()

    # m.terminate()
    for x in range(1, 4):
        m.shovel(value=True, dt=pendulum.DateTime(2020, 1, 1, 0, 0, x))
        # m.shovel(value=True, dt=pendulum.DateTime(2020, 1, 1, 0, 0, x-1))
        m.dump_state_of_the_onion()

    for x in range(3):
        m.shovel(value=False, dt=pendulum.DateTime(2020, 1, 2, 0, 0, x))
        m.dump_state_of_the_onion()

    m.terminate()
    print(list(m.yield_stretches_with(True)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
GIT
---

Helper functions for interacting with
`git <http://git-scm.com/>`_.
"""
from __future__ import print_function
from past.builtins import cmp
from builtins import zip
import re
import logging
from subprocess import PIPE, Popen
import copy

#: regular expression for 'official' tags
OFFICIAL_TAG_RE = re.compile(r'^(?P<major>\d+)\.(?P<minor>\d+)'
                             r'\.(?P<patch>\d+)(\.(?P<build>\d+))?'
                             r'(\-(?P<git_describe>[\d\w\-]+))?$')

#: regular expression for changelog entries
LOG_RE = re.compile(r'^\[(?P<tag>.*?)\] (?P<hash>[\w\d]+) '
                    r'(?P<date>\d{4}\-\d{2}\-\d{2}) \[(?P<author>.*?)\] '
                    r'(?P<msg>.*?)$')

#: regular expression for entries to be considered as 'junk'
MSG_JUNK_RE = re.compile(r'^git\-svn\-id\:|^[\s\.]+$')

#: changelog file lines format
OUTPUT_LOG_FORMAT = "%(date)s %(author)s %(ver)-9s %(hash)s %(msg)s"

#: version portions
PORTIONS = ["major", "minor", "patch", "build"]

LOG = logging.getLogger(__name__)


def git_current_tag(raw=False):
    """
    Retrieve current git tag.

    :param raw: return entire tag as string
    :return: current tag
    :rtype: list
    :rtype: str

    >>> len(git_current_tag())
    1
    >>> git_current_tag(True) == git_current_tag()[0]
    True
    """
    lines = Popen("git describe",
                  shell=True, stdout=PIPE).communicate()[0].decode('utf-8').split("\n")
    if raw is True:
        return lines[0]
    return lines[0].split("-", 1)


def git_tag_tuple(line=None):
    """
    Retrieve current git tag (as tuple).

    :param line: version string
    :return: version information or None
    :rtype: tuple

    >>> git_tag_tuple("") is None
    True
    >>> git_tag_tuple("1.2.3")
    (1, 2, 3, 0)
    """
    m = re.match(OFFICIAL_TAG_RE, line)
    if m:
        gdict = git_tag_dict(line)
        return (gdict.get("major"), gdict.get("minor"), gdict.get("patch"),
                gdict.get("build"))
    return None


def git_tag_dict(line=None):
    """
    Parse an 'official' git tag and return a ``dict()`` containing retrieved
    values.

    Returns an empty dict if data could not be parsed.

    :param line: version string
    :return: version information or empty dict
    :rtype: dict

    >>> dd1 = git_tag_dict("")
    >>> dd1.get("major") is None
    True
    >>> dd2 = git_tag_dict("1.2.3")
    >>> dd2.get('major')
    1
    >>> dd2.get('build')
    0
    >>> dd3 = git_tag_dict("0.0.0.1")
    >>> dd3.get("build")
    1
    >>> dd3.get("major") == dd3.get("minor") == dd3.get("patch") == 0
    True
    >>> dd4 = git_tag_dict()
    >>> dd4.get("major") is not None
    True
    """
    if line is None:
        line = git_current_tag()[0]

    m = re.match(OFFICIAL_TAG_RE, line)
    if m:
        gdict = m.groupdict()
        for key in PORTIONS:
            val = gdict.get(key, 0)
            if val is None:
                val = 0
            gdict[key] = int(val)
        return gdict
    return dict()


def git_official_tags():
    otags = list()

    lines = Popen("git tag -l", shell=True,
                  stdout=PIPE).communicate()[0].split("\n")
    for line in lines:
        tag_tuple = git_tag_tuple(line)
        if tag_tuple:
            otags.append(tag_tuple)
    return list(reversed(sorted(otags)))


def git_official_tags_raw():
    otags = list()

    lines = Popen("git tag -l", shell=True,
                  stdout=PIPE).communicate()[0].split("\n")
    for line in lines:
        m = re.match(OFFICIAL_TAG_RE, line)
        if m:
            gdict = m.groupdict()
            otags.append(gdict)
    return list(reversed(sorted(otags)))


def git_tag_string(tag_dict, plain_int=False):
    """
    Normalise version information in *tag_dict*.

    :param tag_dict: version information
    :return: normalised version string
    :rtype: str

    >>> git_tag_string(git_tag_dict("1.2.15.99"))
    '1.2.15.099'
    >>> git_tag_string(git_tag_dict("1.2.15.99"), plain_int=True)
    '1.2.15.99'
    >>> git_tag_string(git_tag_dict("1.2.0"))
    '1.2.0'
    """
    for key in PORTIONS:
        val = tag_dict.get(key)
        if val is not None:
            val = int(val)
        tag_dict[key] = val

    parts = ["{major:d}.{minor:d}.{patch:d}".format(**tag_dict)]
    if tag_dict.get("build"):
        parts.append(
            (plain_int and "{build:d}" or "{build:03d}").format(**tag_dict))
    return '.'.join(parts)


def git_official_tag_property(tag_tuple):
    for (key, val) in zip(('major', 'minor', 'patch', 'build'), tag_tuple):
        print("build.{:s}={:d}".format(key, val))


def cmp_versions(version_a, version_b, verbose=0):
    """
    Compare two version tags.
    (stolen from renovatio_selenium)

    DANGER, WILL ROBINSON:
    *build* portion is ignored.

    :param version_a: version dict 1
    :param version_b: version dict 2
    :param verbose: verbosity indicator
    :return: comparison result
    :rtype: int

    >>> a035 = {'major': 0, 'minor': 3, 'patch': 5}
    >>> b036 = {'major': 0, 'minor': 3, 'patch': 6}
    >>> c035 = {'major': 0, 'minor': 3, 'patch': 5}
    >>> cmp_versions(a035, b036)
    -1
    >>> cmp_versions(b036, a035)
    1
    >>> cmp_versions(a035, c035)
    0
    >>> x04 = {'major': 0, 'minor': 4}
    >>> cmp_versions(a035, x04)
    -1
    >>> y100 = {'major': 1, 'minor': 0, 'patch': 0}
    >>> cmp_versions(a035, y100)
    -1
    >>> cmp_versions(b036, y100)
    -1
    >>> cmp(1, 2)
    -1
    >>> cmp(2, 1)
    1
    >>> cmp_versions(git_tag_dict('17.3.22.2'), git_tag_dict('16.10.10'))
    1
    """
    if verbose > 1:
        try:
            fmt_string = "%(major)04d.%(minor)04d.%(patch)04d"
            expected = fmt_string % version_a
            actual = fmt_string % version_b
            result = cmp(actual, expected)
        except KeyError:
            result = 23
        LOG.debug("{:s} VS {:s}: {:>3d}".format(version_a, version_b, result))

    key_list = ['major', 'minor', 'patch']
    cmp_value_a = 0
    cmp_value_b = 0

    i = 0
    pot = 100
    for key in reversed(key_list):
        value_a = 0
        value_b = 0

        try:
            value_a = int(version_a.get(key, 0))
        except ValueError:
            pass
        try:
            value_b = int(version_b.get(key, 0))
        except ValueError:
            pass

        if verbose > 3:
            LOG.debug("[a] {:s}={:>3d} ({:>5d})".format(key, value_a,
                                                        value_a * pot ** i))
            LOG.debug("[b] {:s}={:>3d} ({:>5d})".format(key, value_b,
                                                        value_b * pot ** i))

        i += 1
        cmp_value_a += value_a * pot ** i
        cmp_value_b += value_b * pot ** i

    i_result = cmp(cmp_value_a, cmp_value_b)
    LOG.debug("{!s} VS {!s}: {:>3d}".format(version_a, version_b, i_result))
    return i_result


def format_changelog(lines, max_version=None):
    prev_content = None
    log_map = dict()

    for line in lines:
        line = line.strip()
        m = LOG_RE.match(line)
        try:
            gdict = m.groupdict()
        except AttributeError:
            LOG.warning("#BAD: {:s}".format(line))
            continue

        content = gdict.get("msg")
        if prev_content == content:
            LOG.debug("#SKIP/DUPLICATE: {:s}".format(line))
            continue
        prev_content = content

        tag_matcher = re.match(OFFICIAL_TAG_RE, gdict.get("tag"))
        if tag_matcher is None:
            LOG.debug("#SKIP/TAG: {:s}".format(line))
            continue

        if re.match(MSG_JUNK_RE, gdict.get("msg")):
            LOG.debug("#SKIP/JUNK: {:s}".format(line))
            continue

        tag_matcher_gdict = tag_matcher.groupdict()

        for key in PORTIONS:
            val = tag_matcher_gdict.get(key, 0)
            if val is None:
                val = 0
            gdict[key] = int(val)

        gdict['ver'] = "%(major)d.%(minor)d.%(patch)d.%(build)03d" % gdict
        sort_key = "%(major)04d.%(minor)04d.%(patch)04d.%(build)04d" % gdict
        try:
            log_map[sort_key].append(gdict)
        except KeyError:
            log_map[sort_key] = [gdict]

    tags = list(reversed(sorted(log_map.keys())))

    for key in tags:
        if max_version and len(log_map[key]) > 0:
            some_tag_version = log_map[key][0]
            if cmp_versions(max_version, some_tag_version) == -1:
                LOG.error("Too recent: {!r}".format(some_tag_version['ver']))
                continue

        print("[{:s}]".format(log_map[key][0]['tag']))
        for log_data in log_map[key]:
            print(OUTPUT_LOG_FORMAT % log_data)
        print("")


def alter_tag(current_tag, **increment):
    """
    Alter tag values given by *increment*.

    :param current_tag: current tag dictionary
    :param increment: changeset
    :return: altered copy of *current_tag*
    :rtype: dict

    >>> git_tag_string(alter_tag(git_tag_dict("1.2.15.99")))
    '1.2.15.099'
    >>> git_tag_string(alter_tag(git_tag_dict("1.2.15.99"), build=1))
    '1.2.15.100'
    >>> alter_tag(git_tag_dict("1.2.15.99"), build=-199)
    Traceback (most recent call last):
        ...
    AssertionError
    >>> a035 = {'major': 'x', 'minor': 3, 'patch': 5}
    >>> alter_tag(a035, major=1)
    Traceback (most recent call last):
        ...
    TypeError: Can't convert 'int' object to str implicitly
    """
    next_tag = copy.copy(current_tag)

    for key in PORTIONS:
        next_tag[key] = current_tag.get(key, 0) + increment.get(key, 0)
        assert next_tag[key] >= 0

    return next_tag


if __name__ == '__main__':
    import doctest

    (FAILED, SUCCEEDED) = doctest.testmod()
    print("[doctest] SUCCEEDED/FAILED: {:d}/{:d}".format(SUCCEEDED, FAILED))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from future import standard_library

standard_library.install_aliases()
import os
import hashlib
import io as StringIO
import json
import codecs
import platform
import subprocess
import argparse
import logging
import re

import six
from coshed.tools import load_json, next_best_specification_source
from coshed.sassy_glue import SassyController

PATTERN_VALID_APP_NAME = r'[a-z]+[a-z0-9\-\_]*[a-z0-9]+'
REGEX_VALID_APP_NAME = re.compile(PATTERN_VALID_APP_NAME, re.I)

LOG = logging.getLogger(__name__)


def combine(sources, root_path, trunk=None):
    included_sources = 0
    if isinstance(sources, six.string_types):
        sources = [sources]

    sources = [os.path.abspath(os.path.join(root_path, x)) for x in sources]
    folder = os.path.dirname(sources[0])
    b_name0 = os.path.basename(sources[0])
    trunk0, ext = os.path.splitext(b_name0)

    if trunk is None:
        trunk = trunk0

    LOG.info("Combining ...")
    LOG.debug(" folder: {!s}".format(folder))
    LOG.debug(" trunk : {!s}{!s}".format(trunk0, ext))

    content = StringIO.StringIO()
    hash_cookie = hashlib.sha1()
    source_end = "\n"

    for source in sources:
        if not os.path.exists(source):
            LOG.error("Source missing: {!s}".format(source))
            continue
        LOG.debug("  + {!s}".format(source))
        included_sources += 1

        with codecs.open(source, "rb", 'utf-8') as src:
            chunk = src.read(1024).replace('sourceMappingURL', '')
            while chunk:
                content.write(chunk)
                hash_cookie.update(chunk.encode('utf-8'))
                chunk = src.read(1024).replace('sourceMappingURL', '')

        try:
            content.write(source_end)
        except TypeError:
            content.write(unicode(source_end))
        hash_cookie.update(source_end.encode('utf-8'))

    filename = '{trunk}-{hexdigest}{ext}'.format(
        trunk=trunk, hexdigest=hash_cookie.hexdigest(), ext=ext)
    target = os.path.join(folder, filename)
    content.seek(0)

    if included_sources:
        LOG.info(" ... {!s}".format(target))

        with codecs.open(target, "wb", 'utf-8') as tgt:
            tgt.write(content.read())
    else:
        LOG.info(" ... NO CONTENT.")
        raise ValueError("No content")

    return os.path.relpath(target, root_path).replace("\\", '/')


def remove_old_combined(index_path, root_path):
    with codecs.open(index_path, "rb", 'utf-8') as src:
        source_specification = json.load(src)

    delete_me = list()

    for root in source_specification:
        for item_key in source_specification[root]:
            items = source_specification[root][item_key]

            if isinstance(items, six.string_types):
                items = [items]

            delete_me += items

    for item_rel in delete_me:
        item = os.path.abspath(os.path.join(root_path, item_rel))
        # print(item)
        if not os.path.isfile(item):
            continue

        os.unlink(item)


def export_index(index_path, assets):
    with codecs.open(index_path, "wb", 'utf-8') as tgt:
        json.dump(assets, tgt, indent=2, sort_keys=True)


def bundle(source_specification, index_path=None, app_name=None):
    assets = dict()
    root_path = source_specification['_static']
    sassy_path = source_specification.get('_sassy_path')

    if index_path is None:
        index_path = os.path.join(root_path, "index.json")

        if app_name:
            index_path = os.path.join(
                root_path, "index.{:s}.json".format(app_name))

    LOG.info("'static files' root path: {!s}".format(
        source_specification['_static']))
    LOG.info("index path: {!s}".format(index_path))
    LOG.info("sassy path: {!s}".format(sassy_path))

    if sassy_path and os.path.isdir(sassy_path):
        LOG.info("sassy path: {!s}".format(sassy_path))
        sc = SassyController(sassy_path, os.path.join(root_path, 'css'))
        try:
            sc.run(force=source_specification.get('_force', False))
        except Exception as exc:
            LOG.warning("Failed SASS compile!")
            LOG.warning(exc)

    if os.path.isfile(index_path):
        remove_old_combined(index_path, root_path=root_path)

    for root in source_specification:
        asset_items = dict()

        if root.startswith('_'):
            continue

        for item_key in source_specification[root]:
            trunk = item_key
            items = source_specification[root][item_key]

            if app_name:
                trunk = '.'.join((item_key, app_name))

            try:
                items_combined = combine(items,
                                         trunk=trunk, root_path=root_path)
            except ValueError as vexc:
                LOG.warning(vexc)
                continue

            asset_items[item_key] = [items_combined]

        assets[root] = asset_items

    export_index(index_path, assets)

    return assets


def cli_stub(**kwargs):
    app_name = None
    description = "Web application bundling"
    defaults = dict(
        source_path=os.path.join(
            os.getcwd(), 'static/wolfication_specification.json'
        ),
        uwsgi_config_path=os.path.join(os.getcwd(), 'contrib/uwsgi-emperor/'),
        sassy_path=os.path.join(os.getcwd(), 'sassy'),
    )
    kw_keys = ("source_path", "uwsgi_config_path", "index_path" ,"sassy_path")
    for kw_key in kw_keys:
        val = kwargs.get(kw_key)
        if val is not None:
            defaults[kw_key] = val

    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('app_name', metavar='APP',
                        help='web application name', nargs='?')

    # parser.add_argument(
    #     '-n', '--dry-run', action='store_true',
    #     dest="dry_run",
    #     default=False, help="Dry run mode")

    parser.add_argument(
         '-f', '--force', action='store_true',
         dest="force",
         default=False, help="force mode (overwrites stuff etc.)")

    parser.add_argument(
        '-s', '--specification-source', default=defaults['source_path'],
        help="Sources specification. Default: %(default)s", dest="source_path",
        metavar="PATH"
    )

    parser.add_argument(
        '-a', '--sassy', default=defaults['sassy_path'],
        help="Sassy sources. Default: %(default)s", dest="sassy_path",
        metavar="PATH"
    )

    parser.add_argument(
        '-i', '--assets-index', default=None,
        help="Assets index path", dest="index_path",
        metavar="PATH"
    )

    parser.add_argument(
        '-u', '--uwsgi-configuration-path',
        default=defaults['uwsgi_config_path'],
        help="uWSGI configuration path. Default: %(default)s",
        dest="uwsgi_config_path", metavar="PATH"
    )

    cli_args = parser.parse_args()

    source_specification_path = next_best_specification_source(
        cli_args.source_path, app_name=cli_args.app_name,
    )
    LOG.info("Using source specification {!r}".format(source_specification_path))

    source_specification = load_json(source_specification_path)
    try:
        source_specification['_static']
    except KeyError:
        source_specification['_static'] = os.path.dirname(cli_args.source_path)

    source_specification['_force'] = cli_args.force,
    source_specification['_sassy_path'] = cli_args.sassy_path

    if cli_args.app_name:
        if re.match(REGEX_VALID_APP_NAME, cli_args.app_name):
            app_name = cli_args.app_name
        else:
            LOG.warning("Invalid app name {!r}, "
                        "valid names regular expression is {!s}".format(
                cli_args.app_name, PATTERN_VALID_APP_NAME))
            raise ValueError("Invalid app name")

    bundle(source_specification,
           index_path=cli_args.index_path, app_name=app_name)

    if os.path.isdir(cli_args.uwsgi_config_path):
        uwsgi_files = list()

        for item in os.listdir(cli_args.uwsgi_config_path):
            uwsgi_files.append(item)

        if platform.system() != 'Linux':
            LOG.warning("Crippled platform, skipping uWSGI magic")
        else:
            for uw_file in uwsgi_files:
                args = [
                    'touch',
                    os.path.abspath(
                        os.path.join(cli_args.uwsgi_config_path, uw_file))
                ]
                subprocess.call(args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json

site_conf = """server {
    access_log /var/log/nginx/{{ app_name }}-access.log combined;
    error_log  /var/log/nginx/{{ app_name }}-error.log info;

    set $app_root {{ app_root }};
    set $static {{ static_folder|default('$app_root/static') }};

    listen      {{ listen_port }};

    root $static;

    charset     utf-8;
    client_max_body_size 12M;
    client_body_buffer_size 2M;

    # serve static files
    location /static {
        alias $static;
        include snippets/enable_cors;
        access_log off;

        expires +14d;
        add_header Cache-Control public;
    }

    location / {
        include snippets/enable_cors;
        include uwsgi_params;
        uwsgi_pass unix:/opt/uwsgi-sockets/{{ app_name }};
    }

    # Enable gzip but do not remove ETag headers
    gzip on;
    gzip_vary on;
    gzip_comp_level 4;
    gzip_min_length 256;
    gzip_proxied expired no-cache no-store private no_last_modified no_etag auth;
    gzip_types application/javascript application/json text/javascript text/css text/html;
}
"""

app_ini = """[uwsgi]
processes = 2
uid = www-data

#application's base folder
base = {{ app_root }}
chdir = %(base)

# python module to import
module = {{ app_name }}

# flask instance
callable = app

virtualenv = %(base)/venv

socket = /opt/uwsgi-sockets/%n
chmod-socket    = 666

logto = /var/log/uwsgi/vassals/%n.log
plugins = {{ uwsgi_plugin|default('python3') }}
"""

skeletor_py = '''#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Web app skeleton
"""
from __future__ import absolute_import
from builtins import range

import os
import logging

from flask import Flask, abort, render_template, request
from flask import send_from_directory, url_for
import pendulum

from coshed.vial import JINJA_FILTERS
from coshed.tools import load_json
from coshed.flask_tools import wolfication
from coshed.vial import AppResponse

STATIC = '{{ static_folder }}'

BRAND_NAME = '{{ brand_name }}'

APP_NAME = '{{ app_name }}'

ASSETS_JSON = '{{ index_file | default("index.json")}}'

DEBUG_FLAG = True

#: logger instance
LOG = logging.getLogger(APP_NAME)

#: flask application instance
app = wolfication(
    Flask(__name__, static_folder=STATIC),
    jinja_filters=JINJA_FILTERS, app_name=APP_NAME)

assets = load_json(os.path.join(STATIC, ASSETS_JSON))


@app.route('/favicon.ico')
def favicon_handler():
    return send_from_directory(STATIC, 'favicon.ico')


@app.route('/')
def root_handler():
    data = AppResponse(
        assets=assets,
        title="skeletor",
        brand_name=BRAND_NAME,
    )

    return data.flask_obj()


if __name__ == '__main__':
    port = int('{{ listen_port }}')
    bind_address = '0.0.0.0'

    app.run(host=bind_address, port=port, debug=DEBUG_FLAG)
'''

requirements_txt = """requests>=2.13.0
pendulum==2.0.5
future==0.18.2
six==1.13.0
Jinja2==2.10.3
Flask==1.1.1
Flask-Compress==1.4.0
Flask-Cors==3.0.7
coshed>=0.5.0
cachelib==0.1
"""

gitignore = """.DS_Store
._*
*.py[cod]
/.idea
*~
.sass-cache/
*.css.map
/cosh.json
/dist
/MANIFEST
/scss
/js
/venv
/venv3
/*.egg-info
"""

cfignore = """/contrib
"""

spec_empty = {
    "css": {
        "lib": [
            "css/lib_dummy.css"
        ],
        "app": [
            "css/app_dummy.css"
        ]
    },
    "js": {
        "lib": [
            "js/lib_dummy.js",
        ],
        "app": [
            "js/app_dummy.js"
        ]
    }
}

wolfication_specification = json.dumps(spec_empty, indent=2, sort_keys=True)

TEMPLATES_CONTENT = {
    '{templates_root}/nginx/site.conf.jinja2': site_conf,
    '{templates_root}/uwsgi-emperor/app.ini.jinja2': app_ini,
    '{templates_root}/skeletor.py': skeletor_py,
    '{project_root}/requirements.txt': requirements_txt,
    '{project_root}/.gitignore': gitignore,
    '{project_root}/.cfignore': cfignore,
    '{static_folder}/wolfication_specification.{app_name}.json': wolfication_specification,
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

COSH_FILE_DEFAULT = 'cosh.json'

PROJECT_ROOT = os.getcwd()

SCSS_ROOT = os.path.join(PROJECT_ROOT, 'scss')
JS_ROOT = os.path.join(PROJECT_ROOT, 'js')
SCRIPTS_D_ROOT = os.path.join(PROJECT_ROOT, 'contrib/cosh_scripts.d')

#: css-html-js-minify script path
CSS_HTML_JS_MINIFY_BINARY = "css-html-js-minify.py"

#: closure-compiler path
CLOSURE_COMPILER_BINARY = "closure-compiler"

#: default environment key to configuration key mappings
ENV_MAP = [
    ("COSH_SCSS", "scss"),
    ("COSH_INOTIFYWAIT", 'inotifywait'),
    ("COSH_CSS_HTML_JS_MINIFY", CSS_HTML_JS_MINIFY_BINARY),
    ("COSH_CLOSURE_COMPILER", CLOSURE_COMPILER_BINARY),
]

#: default configuration values
COSHED_CONFIG_DEFAULTS = dict(
    #: default scss arguments
    scss_args=[
        "-t compressed",
        "--unix-newlines",
        "--sourcemap=none"
    ],
    #: default inotifywait arguments
    inotifywait_args=[
        "-r",
        "-e modify"
    ],
    #: root path being watched by inotifywait
    watched_root=SCSS_ROOT,
    #: list of tuples: source (SCSS) -> target (CSS) locations
    scss_map=[
        # ('x.scss', 'y.css'),
    ],
    #: list of paths: Javascript files to concatenate
    concat_js_sources=[],
    concat_js_trunk=os.path.join(JS_ROOT, 'lib.bundle.js'),
    concat_js_compiler="closure",
    #: default locations of used binaries
    inotifywait="inotifywait",
    scss="scss",
    #: functions to be called when a change in *watched_root* is detected
    onchange=["call_scss", 'call_js', 'call_scripts'],
    #: path where scripts are located which shall be called on changes to
    #: *watched_root*
    scripts_d=SCRIPTS_D_ROOT,
    css_html_js_minify=CSS_HTML_JS_MINIFY_BINARY,
    closure_compiler=CLOSURE_COMPILER_BINARY,
    #: keys for which relative paths to *coshfile* are enforced
    cosh_force_relative_paths=[
        "watched_root",
        "scripts_d",
        "concat_js_sources",
        "concat_js_trunk",
        "scss_map"
    ],
    rsync_source="",
    rsync_destination="",
    rsync_exclude_from="",
    rsync_args=[
        "-avrn"
    ]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
import os
import sys
import datetime
import json
import time
from wsgiref.handlers import format_date_time
import pprint
from functools import wraps

import flask
from flask import render_template, request
try:
    from werkzeug.contrib.cache import SimpleCache
except ImportError:
    from cachelib import SimpleCache

import pendulum

import coshed

API_VERSION = os.environ.get("API_VERSION")

if API_VERSION is None:
    try:
        API_VERSION = coshed.__version__
    except Exception:
        API_VERSION = '0.0.42'

cache = SimpleCache()

NAVIGATION_ITEMS = [
    ('/', 'Home'),
    ('/doc', 'Documentation'),
]

WANT_DEV = ('i-like-my-sugar-with', 'coffee-and-cream')


def drop_dev(want_dev=None):
    if want_dev is None:
        want_dev = WANT_DEV

    try:
        (arg_name, expectation) = want_dev
        return not request.args.get(arg_name) == expectation
    except Exception:
        pass

    return True


def generate_expires_header(expires=False):
    """
    Generate HTTP expiration header.

    Args:
        expires: expiration in seconds or False for *imediately / no caching*

    Returns:
        dict: key/value pairs defining HTTP expiration information
    """
    headers = {}

    if expires is False:
        headers['Cache-Control'] = 'no-store, no-cache, must-revalidate, ' \
                                   'post-check=0, pre-check=0, max-age=0'
        headers['Expires'] = '-1'
    else:
        now = datetime.datetime.now()
        expires_time = now + datetime.timedelta(seconds=expires)
        headers['Cache-Control'] = 'public'
        headers['Expires'] = format_date_time(
            time.mktime(expires_time.timetuple()))

    return headers


class AppResponse(dict):
    """
    Container class for chalice app responses.
    """

    def __init__(self, *args, **kwargs):
        self.drop_dev = kwargs.get("drop_dev")

        try:
            del kwargs['drop_dev']
        except Exception:
            pass

        dict.__init__(self, *args, **kwargs)

        if '_dev' not in self:
            self['_dev'] = dict()

        try:
            self['version'] = API_VERSION
        except NameError:
            self['version'] = '0.0.42'
        self['python_version'] = sys.version_info.major
        self['_now'] = pendulum.now(tz='UTC').strftime('%Y-%m-%d %H:%M:%S')

        if 'navigation' not in list(self.keys()):
            nav_items = list(self.navigation)
            if nav_items:
                self['navigation'] = nav_items

    @property
    def navigation(self):
        for data in NAVIGATION_ITEMS:
            try:
                (i_url, i_label, whitelist_key) = data
            except ValueError:
                whitelist_key = None
                (i_url, i_label) = data

            if whitelist_key and whitelist_key not in self.get("whitelist", []):
                continue
            yield i_url, i_label

    def flask_obj(self, status_code=200, drop_dev=None, with_version=True,
                  expires=None, headers=None, not_to_be_exposed=None):
        """
        Generate a :py:class:`flask.Response` object for current application
        response object.

        Args:
            status_code (int): HTTP status code, default 200
            drop_dev (bool): Drop development information
            with_version (bool): include version information
            expires: expiration specification
            headers (dict): headers
            not_to_be_exposed (iterable, optional): key/value pairs not to be exposed

        Returns:
            flask.Response: HTTP response object
        """
        del_keys = []

        if drop_dev is None:
            drop_dev = self.drop_dev

        if drop_dev:
            del_keys.append('_dev')

        if with_version is False:
            del_keys.append('version')

        if not_to_be_exposed:
            for ntbe in not_to_be_exposed:
                del_keys.append(ntbe)

        for del_key in del_keys:
            try:
                del self[del_key]
            except KeyError:
                pass

        if headers is None:
            headers = dict()

        headers["Content-Type"] = "application/json"

        if expires is not None:
            headers.update(generate_expires_header(expires))

        try:
            body = json.dumps(self, indent=2, sort_keys=True)
        except TypeError:
            body = pprint.pformat(self)
            headers["Content-Type"] = "text/plain"
            status_code = 500

        return flask.Response(
            status=status_code,
            headers=headers,
            response=body
        )


def request_wants_mimetype(mtb, other='text/html'):
    best = request.accept_mimetypes.best_match([mtb, other])
    return best == mtb and request.accept_mimetypes[best] > \
                           request.accept_mimetypes[other]


def request_wants_json():
    return request_wants_mimetype('application/json')


def cached(timeout=5 * 60, key='view/%s'):
    def decorator(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            cache_key = key % request.path
            rv = cache.get(cache_key)
            if rv is not None:
                return rv
            rv = f(*args, **kwargs)
            cache.set(cache_key, rv, timeout=timeout)
            return rv

        return decorated_function

    return decorator


def cache_render_template(template_name, **context):
    expires_seconds = 60 * 60
    cache_key = template_name

    if context.get("expires"):
        expires_seconds = context.get("expires")
        del context["expires"]

    if context.get("cache_key"):
        cache_key = context.get("cache_key")
        del context["cache_key"]

    try:
        payload = cache.get(cache_key)
        if payload is None:
            raise KeyError(cache_key)
    except KeyError:
        payload = render_template(template_name, **context)
        cache.set(cache_key, payload, timeout=expires_seconds)

    # print payload, cache_key

    return payload


def format_version_tag(value):
    try:
        (base, _) = value.split('+', 1)
        return '{:s}*'.format(base)
    except ValueError:
        return value


def get_request_url(cf_port):
    """
    Hackfix: if we are on cloud foundry we always want HTTPS

    Returns:
        str: request URL
    """
    if cf_port is not None:
        return request.url.replace("http://", "https://")

    return request.url


JINJA_FILTERS = dict(
    # range_datetime=format_range_datetime,
    # on_off=format_on_off,
    # local_dt=format_local_dt,
    version_tag=format_version_tag,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from builtins import object
import os
import subprocess
import logging
import glob

from coshed.coshed_concat import CoshedConcatMinifiedJS, CoshedConcatClosure


class CoshedWatcher(object):
    def __init__(self, cosh_cfg):
        self.cosh_config_obj = cosh_cfg
        self.log = logging.getLogger(__name__)

    def call_scss(self):
        if not self.cosh_config_obj.scss_map:
            self.log.debug("Empty scss_map!")

        for (src, dst) in self.cosh_config_obj.scss_map:
            scss_call = '{binary} {args} "{src}":"{dst}"'.format(
                binary=self.cosh_config_obj.scss,
                args=' '.join(self.cosh_config_obj.scss_args),
                src=src, dst=dst)
            self.log.info(" {!s}".format(scss_call))
            scss_rc = subprocess.call(scss_call, shell=True)
            self.log.info("# RC={!s}".format(scss_rc))

    def call_js(self):
        if not self.cosh_config_obj.concat_js_sources:
            self.log.debug("Empty concat_js_sources!")
        if self.cosh_config_obj.concat_js_compiler == 'closure':
            cat = CoshedConcatClosure(
                self.cosh_config_obj.concat_js_sources,
                self.cosh_config_obj.concat_js_trunk,
                closure_binary=self.cosh_config_obj.closure_compiler
            )
        else:
            cat = CoshedConcatMinifiedJS(
                self.cosh_config_obj.concat_js_sources,
                self.cosh_config_obj.concat_js_trunk,
                css_html_js_minify=self.cosh_config_obj.css_html_js_minify
            )
        cat.write()

    def call_scripts(self):
        try:
            self.cosh_config_obj.scripts_d
        except AttributeError:
            self.log.debug("no scripts_d attribute")
            return

        glob_scripts = u'{:s}/*'.format(self.cosh_config_obj.scripts_d)

        for s_filename in glob.glob(glob_scripts):
            if s_filename.endswith("~"):
                continue
            if os.path.basename(s_filename).startswith("OFF."):
                continue
            if not os.access(s_filename, os.X_OK):
                continue

            command = u'{s_filename} {coshfile}'.format(
                s_filename=s_filename, coshfile=self.cosh_config_obj.coshfile)
            self.log.info(" {!s}".format(command))
            command_rc = subprocess.call(command, shell=True)
            self.log.info("# RC={!s}".format(command_rc))

    def on_change(self):
        for func_name in self.cosh_config_obj.onchange:
            # self.log.debug("About to call {:s}".format(func))
            try:
                func = getattr(self, func_name)
                func()
            except AttributeError:
                self.log.warning(
                    "Missing method {!r}. IGNORED.".format(func_name))

    def watch(self):
        root = os.path.abspath(self.cosh_config_obj.watched_root)
        self.log.info("Watching {!s}".format(root))
        inotifywait_call = '{binary} {args} "{folder}"'.format(
            binary=self.cosh_config_obj.inotifywait,
            args=' '.join(self.cosh_config_obj.inotifywait_args),
            folder=root)

        rc = subprocess.call(inotifywait_call, shell=True)
        while rc == 0:
            self.on_change()
            rc = subprocess.call(inotifywait_call, shell=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from builtins import object
import os
import sys
import random
import logging
import argparse
import copy
import codecs
import re

from jinja2 import Environment, FileSystemLoader
import six

from coshed.bundy import PATTERN_VALID_APP_NAME
from coshed.bundy import REGEX_VALID_APP_NAME
from coshed.wolfication_template_contents import TEMPLATES_CONTENT

PATH_KEYS = ('project_root', 'app_root', 'static_folder')

TEMPLATE_ARGS_DEFAULTS = dict(
    app_root=os.path.abspath(os.getcwd()),
    brand_name=None,
    project_root=os.path.abspath(os.getcwd()),
)

TEMPLATE_ARGS_DEFAULTS['static_folder'] = os.path.join(
    TEMPLATE_ARGS_DEFAULTS['app_root'], 'static')

CONFIGURATION_FOLDERS = dict(
    uwsgi_vassals='/etc/uwsgi-emperor/vassals',
    nginx_sites_available='/etc/nginx/sites-available',
    nginx_sites_enabled='/etc/nginx/sites-enabled',
)

CONFIGURATION_FOLDERS_CENTOS = dict(
    uwsgi_vassals='/etc/uwsgi.d',
    nginx_conf_d='/etc/nginx/conf.d',
)


class Wolfication(object):
    def __init__(self, app_name, listen_port=None, *args, **kwargs):
        self.log = logging.getLogger(__name__)
        self.dry_run = kwargs.get("dry_run", False)
        self.do_init = kwargs.get("do_init", False)

        if listen_port is None:
            listen_port = random.randint(23000, 60000)
        else:
            listen_port = int(listen_port)

        if not re.match(REGEX_VALID_APP_NAME, app_name):
            self.log.warning("Invalid app name {!r}, "
                             "valid names regular expression is {!s}".format(
                app_name, PATTERN_VALID_APP_NAME))
            raise ValueError("Invalid app name")

        self.template_args = {
            "app_name": app_name,
            "listen_port": listen_port,
            "index_file": 'index.{:s}.json'.format(app_name)
        }
        self.template_args.update(TEMPLATE_ARGS_DEFAULTS)

        for key in kwargs:
            self.template_args[key] = kwargs[key]

        if kwargs.get('environment_root_override'):
            ert = kwargs.get('environment_root_override')
            self.template_args['project_root'] = ert
            self.template_args['app_root'] = ert
            self.template_args['static_folder'] = os.path.join(ert, 'static')

        if self.template_args['brand_name'] is None:
            self.template_args['brand_name'] = app_name

        self.project_root = self.template_args['project_root']
        self.template_args['contrib_root'] = os.path.abspath(
            os.path.join(self.project_root, 'contrib')
        )

        sys.path.insert(0, self.project_root)

        self.templates_root = os.path.abspath(
            os.path.join(self.template_args['contrib_root'], 'templates/webapp')
        )

        self.template_args['templates_root'] = self.templates_root

        self.log.info("")
        self.log.info("-" * 80)
        for key, value in sorted(six.iteritems(self.template_args)):
            self.log.info("{key:30}: {value}".format(key=key, value=value))
        self.log.info("-" * 80)
        self.log.info("")

        self.env = Environment(loader=FileSystemLoader(self.templates_root))

    def skeleton_app(self):
        template = self.env.get_template('skeletor.py')
        return template.render(**self.template_args)

    def nginx_site(self):
        template = self.env.get_template('nginx/site.conf.jinja2')
        return template.render(**self.template_args)

    def uwsgi_vassal(self):
        template = self.env.get_template('uwsgi-emperor/app.ini.jinja2')
        return template.render(**self.template_args)

    def _dump(self, path, content, overwrite=False):
        if os.path.exists(path) and not overwrite:
            raise IOError("existis: {!s}".format(path))

        with codecs.open(path, "wb", 'utf-8') as tgt:
            tgt.write(content)

    def instructor_install(self, spec, configuration_folders=None):
        if configuration_folders is None:
            configuration_folders = CONFIGURATION_FOLDERS

        merged = copy.copy(configuration_folders)

        for key in spec:
            merged[key] = spec[key][0]

        merged['nginx_config_bname'] = os.path.basename(merged['nginx_config'])

        messages = [
            '',
            '#' * 80,
            ' Installation instructions ...',
            '#' * 80,
            'ln -s {uwsgi_config} {uwsgi_vassals}'.format(**merged),
        ]

        if 'nginx_sites_available' in merged:
            messages.append(
                'ln -s {nginx_config} {nginx_sites_available}'.format(
                    **merged))
            messages.append(
                'ln -s '
                '{nginx_sites_available}/{nginx_config_bname} '
                '{nginx_sites_enabled}'.format(**merged))
        else:
            messages.append(
                'ln -s {nginx_config} {nginx_conf_d}'.format(**merged))

        messages.append('')
        for msg in messages:
            self.log.info(msg)

    def instructor_initialisation(self):
        messages = [
            ''
            '~' * 80,
            ' Project initialisation instructions ...',
            '~'  * 80,
            'cd "{project_root:s}" && virtualenv -p python3 venv'.format(
                **self.template_args),
            'git init',
            '. "{project_root:s}/venv/bin/activate"'.format(
                **self.template_args),
            'pip install -r "{project_root:s}/requirements.txt"'.format(
                **self.template_args),
            'coshed-bundy {app_name:s} '
            '&& python "{project_root:s}/{app_name:s}.py"'.format(
                **self.template_args),
            'git add . && git commit -am "initial commit"'
            ]

        for msg in messages:
            self.log.info(msg)

    def _initialise_environment(self):
        roots = (
            'project_root', 'templates_root', 'contrib_root', 'static_folder')

        for root_key in roots:
            root_dir = self.template_args[root_key]

            if not os.path.isdir(root_dir):
                self.log.warning("Missing folder ({!s}) {!s}".format(
                    root_key, root_dir))
                if self.dry_run:
                    self.log.info("WOULD create {!s}".format(root_dir))
                else:
                    os.makedirs(root_dir)

        for rel_path, content in six.iteritems(TEMPLATES_CONTENT):
            abs_path = rel_path.format(**self.template_args)

            if not os.path.isfile(abs_path):
                abs_folder = os.path.dirname(abs_path)

                if self.dry_run:
                    self.log.info("WOULD create {!s}".format(abs_folder))
                else:
                    os.makedirs(abs_folder, exist_ok=True)

            if self.dry_run:
                self.log.info("WOULD write  {!s}".format(abs_path))
            else:
                if not os.path.isfile(abs_path) or self.template_args['force']:
                    with open(abs_path, "w") as tgt:
                        tgt.write(content)
                else:
                    self.log.warning("NOT overwriting {!s}".format(abs_path))

    def persist(self):
        if self.do_init:
            self._initialise_environment()

        spec = dict(
            nginx_config=('{contrib_root}/nginx/{app_name}.conf'.format(
                **self.template_args), self.nginx_site),
            uwsgi_config=('{contrib_root}/uwsgi-emperor/{app_name}.ini'.format(
                **self.template_args), self.uwsgi_vassal),
            app=('{project_root}/{app_name}.py'.format(
                **self.template_args), self.skeleton_app),
        )

        self.log.info(
            "Creating {app_name} on port {listen_port} with "
            "app_root {app_root}".format(**self.template_args))

        if self.dry_run:
            self.log.info("DRY RUN.")

        for target, func in list(spec.values()):
            if self.dry_run:
                self.log.info("WOULD write {!s}".format(target))
                continue

            self.log.debug(target)
            target_parent = os.path.dirname(target)
            if not os.path.isdir(target_parent):
                os.makedirs(target_parent)
            self._dump(target, func(), overwrite=self.template_args['force'])
        self.instructor_install(spec, self.template_args['configuration_folders'])

        if self.do_init:
            self.instructor_initialisation()


def cli_stub():
    description = "Webapp skeleton generator"
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('app_name', metavar='APP',
                        help='web application name')

    parser.add_argument(
        '-n', '--dry-run', action='store_true',
        dest="dry_run",
        default=False, help="Dry run mode")

    parser.add_argument(
        '-f', '--force', action='store_true',
        dest="force",
        default=False, help="force mode (overwrites stuff etc.)")

    parser.add_argument(
        '-p', '--port', default=None, type=int,
        help="listening port", dest="listen_port"
    )

    parser.add_argument(
        '--platform-centos', default=CONFIGURATION_FOLDERS,
        action="store_const",
        const=CONFIGURATION_FOLDERS_CENTOS,
        help="Use CentOS platform configuration defaults",
        dest="configuration_folders"
    )

    params_group = parser.add_argument_group("More Parameters")
    for key in TEMPLATE_ARGS_DEFAULTS:
        params_group.add_argument(
            '--{:s}'.format(key), default=TEMPLATE_ARGS_DEFAULTS[key],
            help="{:s}. Default is %(default)s".format(key),
            dest=key,
            metavar=(key in PATH_KEYS and 'PATH' or None)
        )

    env_group = parser.add_argument_group(
        "Environment Parameters",
        description="Override path parameters with default prefix")

    env_group.add_argument(
        '-i', '--init', action='store_true',
        dest="do_init",
        default=False, help="Initialise environment")

    env_group.add_argument(
        '-e', '--environment-root',
        dest="environment_root_override",
        default=None, help="Override paths for {:s}".format(
            ', '.join(PATH_KEYS)
        ),
        metavar="PATH"
    )

    cli_args = parser.parse_args()

    try:
        wolfication = Wolfication(**vars(cli_args))
    except ValueError as vexc:
        sys.exit(1)

    wolfication.persist()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import json


def load_json(path):
    """
    Load JSON encoded file and return its contents.
    """
    with open(path, "r") as src:
        content = json.load(src)

    return content


def next_best_specification_source(fallback, app_name=None, root_path=None,
                                   trunk=None):
    """
    Determine the best matching wolfication specification for *app_name*.

    Args:
        fallback: fallback source name
        app_name (unicode, optional): application name
        root_path (unicode, optional): path where specification files may be located
        trunk (unicode, optional): specification file trunk

    Returns:
        unicode: specification path
    """
    candidates = list()

    if root_path is None:
        root_path = os.path.dirname(fallback)

    if trunk is None:
        trunk = 'wolfication_specification'

    if app_name:
        candidates.append(
            os.path.join(
                root_path,
                '{trunk}.{app_name}.json'.format(
                    app_name=app_name, trunk=trunk)
            )
        )

    candidates.append(
        os.path.join(
            root_path,
            '{trunk}.json'.format(trunk=trunk)
        )
    )

    for candy in candidates:
        if os.path.isfile(candy):
            return os.path.abspath(candy)

    return fallback


def next_best_asset_source(fallback, app_name=None, root_path=None):
    """
    Determine the best matching wolfication index for *app_name*.

    Args:
        fallback: fallback source name
        app_name (unicode, optional): application name
        root_path (unicode, optional): path where specification files may be located

    Returns:
        unicode: index path
    """
    return next_best_specification_source(fallback, app_name=app_name,
                                          root_path=root_path, trunk="index")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Versioning for git projects CLI tool.
"""
from __future__ import division
from builtins import object
from past.utils import old_div
import logging
import datetime
import subprocess
import argparse
from argparse import RawDescriptionHelpFormatter

import coshed
from coshed.git_changelog_helper import git_tag_dict
from coshed.git_changelog_helper import git_current_tag
from coshed.git_changelog_helper import PORTIONS
from coshed.git_changelog_helper import git_tag_string
from coshed.git_changelog_helper import alter_tag

#: command for adding a bumped up version tag
BUMP_TAG = 'git tag -a "{next_tag}" -m "version bump"'

#: command for pushing changes to target repo
CMD_PUSH_BRANCH = 'git push {target} {branch}'

#: command for pushing tags to target repo
CMD_PUSH_TAGS = 'git push {target} --tags'

VERSIONING_SCHEME_CALVER = 'calver'

VERSIONING_SCHEME_SEMANTIC = 'semantic'

#: Format string for 'current' to 'next' tag bump
CURRENT_NEXT = '''
        Example: current tag {current!r} will be followed by {next!r}.
'''

# Tool description
DESCRIPTION = '''
calve - Versioning for git projects helper using coshed {version!s}

Automatic bumping of version tags based on current date
or existing semantic version tag.

SEMANTIC scheme: MAJOR.MINOR.PATCH[.BUILD]

CALVER scheme: YY.DD.MM[.BUILD]

        The first version bump on 2016-08-31 will yield version   16.8.31,
        the following version bump on the same day will result in 16.8.31.1
        and the next ...                                          16.8.31.2

'''.format(version=coshed.__version__)


def calver_next(use_log=None, **kwargs):
    """
    Generate the next version tag (calendar versioning scheme).

    Args:
        use_log: logger instance
        kwargs: optional arguments

    Returns:
        current and next tag values
    """
    if use_log is None:
        use_log = logging.getLogger(__name__)

    current_tag = git_current_tag()
    vdict = git_tag_dict(current_tag[0])

    year = int(vdict['major'])
    if (old_div(year, 1000)) < 1:
        year += 2000

    dt_prev = datetime.datetime(year=year, month=int(vdict['minor']),
                                day=int(vdict['patch']))
    dt_current = datetime.datetime.now()

    try:
        bumper = int(vdict['build'])
    except TypeError:
        bumper = 0

    use_log.debug('%s', "dt_prev={!r} dt_current={!r} bumper={!r}".format(
        dt_prev, dt_current, bumper))
    use_log.debug('%s', "dt_current.date-dt_prev.date = {!r}".format(
        dt_current.date() - dt_prev.date()))

    dt_next = dt_current
    if dt_prev.date() < dt_current.date():
        bumper = 0

    next_tag = '{:d}.{:d}.{:d}'.format(
        dt_next.year % 100, dt_next.month, dt_next.day)

    if bumper or dt_prev.date() == dt_current.date():
        bumper += 1
        next_tag += '.{:d}'.format(bumper)

    return current_tag, next_tag


def semver_next(use_log=None, cli_args=None):
    """
    Generate the next version tag (semantic versioning scheme).

    Args:
        use_log: logger instance

    Returns:
        current and next tag values
    """
    if use_log is None:
        use_log = logging.getLogger(__name__)

    if cli_args is None:
        raise ValueError("cli_args may not be None!")

    current_tag = git_current_tag()
    vdict = git_tag_dict(current_tag[0])

    increment = dict()
    inc_keys = list()
    for portion_key in PORTIONS:
        key = 'alter_{portion_key}'.format(portion_key=portion_key)
        increment[portion_key] = getattr(cli_args, key)
        if increment[portion_key]:
            inc_keys.append(portion_key)

    use_log.debug('%s', 'increment={!r}'.format(increment))
    use_log.debug('%s', 'inc_keys={!r}'.format(inc_keys))

    if len(inc_keys) == 0:
        increment['build'] = 1
        inc_keys.append('build')
    elif len(inc_keys) > 1:
        raise ValueError("Only one of {!r} may be altered at once".format(
            PORTIONS))

    inc_key = inc_keys[0]
    for key in reversed(PORTIONS):
        if key != inc_key:
            increment[key] = - vdict[key]
        else:
            break

    use_log.debug('%s', 'increment={!r}'.format(increment))
    use_log.debug('%s', 'inc_keys={!r}'.format(inc_keys))

    next_tag = git_tag_string(alter_tag(vdict, **increment))

    return current_tag, next_tag


# try adding example version bump if possible:
try:
    vdict = dict()
    for key in PORTIONS:
        alter_key = 'alter_{key}'.format(key=key)
        vdict[alter_key] = 0
    vdict['alter_patch'] += 1
    fake_cli_args = argparse.Namespace(**vdict)

    current_version_tag, current_next_tag = semver_next(cli_args=fake_cli_args)
    DESCRIPTION += CURRENT_NEXT.format(
        current=current_version_tag[0], next=current_next_tag)
except Exception as dexc:
    log = logging.getLogger(__name__)
    log.debug(dexc)


class Calve(object):
    """
    Versioning for git projects helper
    """

    def __init__(self, *args, **kwargs):
        self.log = logging.getLogger(__name__)
        self.args = kwargs.get('args')
        if self.args is None:
            try:
                self._parse_cli_args()
            except Exception as exc:
                self.log.error(exc)
                self.log.warning("Error parsing CLI arguments")
                self.args = argparse.Namespace()

        if self.args.dry_run:
            self.log.info("DRY RUN ...")

    def _parse_cli_args(self):
        parser = argparse.ArgumentParser(
            description=DESCRIPTION,
            formatter_class=RawDescriptionHelpFormatter)

        parser.add_argument(
            '--dry-run', '-n',
            default=False, help='Dry Run',
            action='store_true', dest="dry_run")

        group = parser.add_argument_group('Versioning Scheme')
        scheme_group = group.add_mutually_exclusive_group()

        scheme_group.add_argument(
            '--semantic', '-s',
            help='Semantic Versioning, default is %(default)s',
            action='store_const', dest="scheme",
            const=VERSIONING_SCHEME_SEMANTIC,
            default=VERSIONING_SCHEME_SEMANTIC)

        scheme_group.add_argument(
            '--calver', '-c',
            help='Calendar Versioning, default is %(default)s',
            action='store_const', dest="scheme",
            const=VERSIONING_SCHEME_CALVER,
            default=VERSIONING_SCHEME_SEMANTIC)

        publish = parser.add_argument_group('Publish Options')

        publish.add_argument(
            '--lazy-bum', '-l',
            default=False, help='Push changes and tag after version bumping',
            action='store_true', dest="lazy_bum")

        publish.add_argument(
            '--lazy-devpi', '-p',
            default=False, help='Upload using devpi after version bumping',
            action='store_true', dest="lazy_devpi")

        publish.add_argument(
            '--target', '-t', metavar="GIT_ORIGIN",
            help='git target. default: %(default)s',
            default='origin',
            dest="git_target")

        publish.add_argument(
            '--branch', '-b', metavar="GIT_BRANCH",
            help='git branch. default: %(default)s',
            default='master',
            dest="git_branch")

        alter = parser.add_argument_group('Tag altering')
        alter.add_argument('--major', '--aa', dest="alter_major", const=1,
                           action='store_const', default=0,
                           help="increment MAJOR version portion")
        alter.add_argument('--minor', '--ai', dest="alter_minor", const=1,
                           action='store_const', default=0,
                           help="increment MINOR version portion")
        alter.add_argument('--patch', '--ap', dest="alter_patch", const=1,
                           action='store_const', default=0,
                           help="increment PATCH version portion")
        alter.add_argument('--build', '--ab', dest="alter_build", const=1,
                           action='store_const', default=0,
                           help="increment BUILD version portion")

        self.args = parser.parse_args()

    def main(self):
        """
        Main Loop
        """
        if self.args.scheme == VERSIONING_SCHEME_CALVER:
            bump_func = calver_next
        elif self.args.scheme == VERSIONING_SCHEME_SEMANTIC:
            bump_func = semver_next
        else:
            raise NotImplementedError(self.args.scheme)

        current_tag, next_tag = bump_func(use_log=self.log, cli_args=self.args)

        self.log.info("Bumping tag from {current_tag} to {next_tag}".format(
            current_tag=current_tag[0], next_tag=next_tag))

        assert current_tag[0] != next_tag
        self._bump(next_tag=next_tag)

        if self.args.lazy_bum:
            self.log.warning("You lazy bum!")
            self._publish()

        if self.args.lazy_devpi:
            self.log.warning("You lazy bum!")
            self._devpi_upload()

    def _call(self, cmd):
        """
        Simple execution of arbitrary commands.

        Args:
            cmd: command

        Returns:
            return code
        """
        self.log.debug(cmd)
        return subprocess.call(cmd, shell=True)

    def _bump(self, next_tag):
        """
        Create a bumped up git tag

        Args:
            next_tag: new tag

        Returns:
            return code
        """
        cmd = BUMP_TAG.format(next_tag=next_tag)

        if self.args.dry_run:
            self.log.info('%s', "WOULD execute {!r}".format(cmd))
            return

        return self._call(cmd)

    def _publish(self):
        """
        Push changes and tag to target repository.

        Returns:
            return codes
        """
        params = {
            'target': self.args.git_target,
            'branch': self.args.git_branch,
        }
        rcs = []

        for cmd_fmt in [CMD_PUSH_BRANCH, CMD_PUSH_TAGS]:
            cmd = cmd_fmt.format(**params)

            if self.args.dry_run:
                self.log.info('%s', "WOULD execute {!r}".format(cmd))
                continue

            rcs.append(self._call(cmd))

        return rcs

    def _devpi_upload(self):
        """
        Upload package using devpi

        Returns:
            return codes
        """
        rcs = []

        cmd = 'devpi upload'

        if self.args.dry_run:
            self.log.info('%s', "WOULD execute {!r}".format(cmd))
            return

        rcs.append(self._call(cmd))

        return rcs


def cli_stub():
    logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)-15s %(levelname)8s %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S'
    )

    cli = Calve()
    cli.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
import sys
import logging
import argparse

import coshed
from coshed.defaults import COSH_FILE_DEFAULT
from coshed.defaults import COSHED_CONFIG_DEFAULTS, ENV_MAP
from coshed.coshed_config import CoshedConfig
from coshed.coshed_watcher import CoshedWatcher

logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s %(levelname)-8s %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')

LOG = logging.getLogger("coshed_watcher")

if __name__ == '__main__':
    argparser = argparse.ArgumentParser(epilog="coshed {:s}".format(
        coshed.__version__))
    argparser.add_argument(
        '--cosh-file', '-f',
        dest="coshfile", default=COSH_FILE_DEFAULT, metavar="PATH",
        help="JSON config file [%(default)s]")
    argparser.add_argument(
        '--force-update', '-u', action='store_true',
        dest="force_update",
        help="Force updating of CSS files and terminate", default=False)
    argparser.add_argument(
        '--verbose', '-v', action='count',
        default=0, dest="verbose",
        help="verbosity (more v: more verbosity)")

    args = argparser.parse_args()

    cosh_cfg = CoshedConfig(
        defaults=COSHED_CONFIG_DEFAULTS,
        coshfile=args.coshfile,
        environ_key_mapping=ENV_MAP,
    )

    LOG.warning("inotifywait and scss binaries need to be installed!")
    LOG.warning(
        " 'apt-get install inotify-tools ruby-sass' on debian "
        "derived distributions")
    LOG.warning("For JavaScript minification css-html-js-minify "
                "needs to be installed.")

    for env_key, key in ENV_MAP:
        LOG.debug(
            "You may use environment variable {env_key!r} to "
            "override configuration key {key!r}.".format(
                env_key=env_key, key=key))

    if args.verbose > 0:
        LOG.debug("Supported S/CSS transformations:")
        for (src, dst) in cosh_cfg.scss_map:
            LOG.debug("{!r} -> {!r}".format(src, dst))

    if args.verbose > 1:
        LOG.info("coshed configuration:")
        LOG.info(cosh_cfg)

    cosh_op = CoshedWatcher(cosh_cfg)

    if args.force_update:
        cosh_op.on_change()
        sys.exit(0)

    try:
        cosh_op.watch()
    except KeyboardInterrupt:
        LOG.info("\nAborted.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from past.builtins import basestring
from builtins import object
import os
import json
import argparse
import codecs

from coshed.defaults import COSH_FILE_DEFAULT


class CoshedConfig(object):
    """
    Simple Configuration object:

        * settings may be stored in a JSON file
        * access settings by attribute or key
        * default values may be set by `dict()`, kwargs and environment variables

    """

    def __init__(self, *args, **kwargs):
        self._data_sources = {
            'environ': dict(),
            'config': dict()
        }
        self._sources_log = []
        self._key_set = None
        key_set = kwargs.get("keys")
        if key_set and isinstance(key_set, set):
            self._key_set = key_set
        coshfile = kwargs.get("coshfile")
        self._ro = kwargs.get("ro", False)
        defaults = kwargs.get("defaults", {})
        environ_key_mapping = kwargs.get("environ_key_mapping", [])

        for source_value in args:
            if isinstance(source_value, argparse.Namespace):
                self._data_sources["args"] = vars(source_value)
            else:
                if coshfile is None:
                    try:
                        # value might be string qualifying for a path value
                        source_value.startswith("-")
                        coshfile = source_value
                    except Exception:
                        pass

        for key in kwargs:
            if key in (
                    'defaults', 'coshfile', 'ro', 'keys',
                    'environ_key_mapping'):
                continue
            source_value = kwargs[key]
            defaults[key] = source_value

        self._data_sources["defaults"] = defaults

        if coshfile is None:
            self._coshfile = COSH_FILE_DEFAULT
        else:
            self._coshfile = os.path.abspath(coshfile)

        if os.path.isfile(self._coshfile):
            with codecs.open(self._coshfile, "rb", 'utf-8') as src:
                try:
                    self._data_sources["config"] = json.load(src)
                except ValueError:
                    pass

        for env_key, key in environ_key_mapping:
            try:
                self._data_sources["environ"][key] = os.environ[env_key]
            except KeyError:
                pass

        self._populate()
        self.write()

    def _acceptable_attr_name(self, attr_name):
        if attr_name.startswith('_') or attr_name == 'coshfile':
            return False
        if self._key_set:
            if attr_name not in self._key_set:
                return False
        if hasattr(self, attr_name) and callable(getattr(self, attr_name)):
            return False

        return True

    def __setitem__(self, key, value):
        setattr(self, key, value)

    def __getitem__(self, item):
        try:
            return getattr(self, item)
        except AttributeError:
            raise KeyError(item)

    def _populate(self):
        self._sources_log = []
        source_order = ('args', 'environ', 'config', 'defaults')
        key_set = set()

        if self._key_set:
            key_set = self._key_set
        else:
            for source_key in source_order:
                try:
                    key_set |= set(self._data_sources[source_key].keys())
                except KeyError:
                    pass

        for key in key_set:
            if not self._acceptable_attr_name(key):
                continue

            for source_key in source_order:
                try:
                    value = self._data_sources[source_key][key]
                    setattr(self, key, value)
                    self._sources_log.append(
                        '{:s}={!r} defined by {:s}'.format(key, value,
                                                           source_key.upper()))
                    break
                except KeyError:
                    pass
        self._force_relative_paths()

    def _rel_path(self, value):
        coshdir = os.path.dirname(self.coshfile)
        abs_path = os.path.abspath(value)
        return os.path.relpath(abs_path, start=coshdir)

    def _force_relative_paths(self):
        if not self.coshfile:
            return

        try:
            self.cosh_force_relative_paths
        except AttributeError:
            return

        for key in self.cosh_force_relative_paths:
            value = self[key]
            rel_value = False

            if isinstance(value, (basestring, str)):
                rel_value = self._rel_path(value)
            elif isinstance(value, list):
                rel_value = list()
                for list_item in value:
                    if isinstance(list_item, (basestring, str)):
                        list_item = self._rel_path(list_item)
                    elif isinstance(list_item, (tuple, list)):
                        list_item = [self._rel_path(x) for x in list_item]
                    rel_value.append(list_item)

            if rel_value is False:
                continue
            self[key] = rel_value

    def _data_dict(self):
        data = dict()
        for key in list(self.keys()):
            data[key] = self[key]
        return data

    def get_filename(self):
        return os.path.abspath(self._coshfile)

    coshfile = property(get_filename)

    def write(self, force=False):
        if self._ro and not force:
            return
        data = self._data_dict()

        if not data:
            return

        if self._data_sources["config"] == data:
            return

        with codecs.open(self._coshfile, "wb", "utf-8") as tgt:
            json.dump(data, tgt, indent=2)

    def keys(self):
        for attr_name in dir(self):
            if self._acceptable_attr_name(attr_name):
                yield attr_name

    def kv_pairs(self):
        for key in list(self.keys()):
            value = getattr(self, key)
            yield key, value

    def __str__(self):
        items = list()
        for attr_name in list(self.keys()):
            items.append((attr_name, getattr(self, attr_name)))
        return '<{:s} {:s}>'.format(self.__class__.__name__, ' '.join(
            ['{:s}={!r}'.format(k, v) for k, v in sorted(items)]))

    def __repr__(self):
        return repr(self._data_dict())


class CoshedConfigReadOnly(CoshedConfig):
    def __init__(self, coshfile):
        CoshedConfig.__init__(self, ro=True, coshfile=coshfile)

    def write(self, *args, **kwargs):
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from builtins import str
from past.builtins import basestring


def to_unicode_or_bust(obj, encoding='utf-8'):
    """
    Convert string *obj* to unicode.
    *obj* is left alone if it is not string instance.
    If *obj* cannot be converted some sort of exception is thrown.

    Args:
        obj (basestring): string to convert
        encoding (str): Target encoding
    """
    if isinstance(obj, basestring):
        if not isinstance(obj, str):
            obj = str(obj, encoding)
    return obj



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os


def next_best_swagger_in_source(swagger_root_path, app_name=None):
    """
    Determine the best matching swagger specification for *app_name*.

    Args:
        swagger_root_path: swagger files path
        app_name: application name

    Returns:
        unicode: swagger specification path

    Raises:
        ValueError: in case no swagger specification file could be found
    """
    swagger_in_candidates = list()

    if app_name:
        swagger_in_candidates.append(
            'swagger.{app_name}.in.json'.format(app_name=app_name))

    swagger_in_candidates.append('swagger.in.json')

    for candy in swagger_in_candidates:
        candy_path = os.path.join(swagger_root_path, candy)
        if os.path.isfile(candy_path):
            return os.path.abspath(candy_path)

    raise ValueError("No swagger.in file found")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import versioneer

setup(
    name='coshed',
    author="doubleO8",
    author_email="wb008@hdm-stuttgart.de",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description='Project deployment and distribution tools for lazy developers',
    long_description="",
    url="https://github.com/doubleO8/coshed",
    packages=['coshed'],
    install_requires=[
        "beautifulsoup4>=4.6.0",
        "requests>=2.13.0",
        "pendulum>=2.0.5",
        "future>=0.18.2",
        "six>=1.13.0",
        "Jinja2>=2.10.3",
        "Flask>=1.1.1",
        "Flask-Compress>=1.4.0",
        "Flask-Cors>=3.0.7",
        "cachelib>=0.1",
    ],
    scripts=['coshed-watcher.py'],
    entry_points={
        'console_scripts': [
            'coshed-bundy = coshed.bundy:cli_stub',
            'coshed-wolfication = coshed.wolfication:cli_stub',
            'coshed-calve = coshed.calving:cli_stub',
        ]
    }
)
